
    /*!
    * @ledecmf/code-editor v0.1.0
    * (c) 2019 HollyIT (limited liability)
    * @license MIT
    */
  
import 'core-js/modules/es6.regexp.replace';
import 'codemirror/addon/edit/matchtags.js';
import 'codemirror/addon/edit/closetag.js';
import CodeMirror from 'codemirror';
import 'codemirror/addon/search/searchcursor';
import 'core-js/modules/es6.regexp.match';
import 'core-js/modules/es6.regexp.constructor';
import iterator$1 from 'core-js/library/fn/symbol/iterator';
import symbol$1 from 'core-js/library/fn/symbol';
import 'core-js/modules/web.dom.iterable';
import 'core-js/modules/es6.regexp.split';
import 'core-js/modules/es6.function.name';
import defineProperty$1 from 'core-js/library/fn/object/define-property';
import 'core-js/modules/es6.string.link';
import { EditorIcon, EditorDropButton, EditorButton } from '@ledecmf/editor-core';

var iterator = iterator$1;

var symbol = symbol$1;

function _typeof2(obj) {
  if (typeof symbol === "function" && _typeof(iterator) === "symbol") {
    _typeof2 = function _typeof2(obj) {
      return _typeof(obj);
    };
  } else {
    _typeof2 = function _typeof2(obj) {
      return obj && typeof symbol === "function" && obj.constructor === symbol && obj !== symbol.prototype ? "symbol" : _typeof(obj);
    };
  }

  return _typeof2(obj);
}

function _typeof(obj) {
  if (typeof symbol === "function" && _typeof2(iterator) === "symbol") {
    _typeof = function _typeof(obj) {
      return _typeof2(obj);
    };
  } else {
    _typeof = function _typeof(obj) {
      return obj && typeof symbol === "function" && obj.constructor === symbol && obj !== symbol.prototype ? "symbol" : _typeof2(obj);
    };
  }

  return _typeof(obj);
}

/**
 * Modified from Wordpress AutoP function.
 *
 * @param text
 * @param blocks
 * @returns {string}
 */
function AutoP (text, options) {
  var preserve_linebreaks = false,
      preserve_br = false,
      blocklist = 'table|thead|tfoot|caption|col|colgroup|tbody|tr|td|th|div|dl|dd|dt|ul|ol|li|pre' + '|form|map|area|blockquote|address|math|style|p|h[1-6]|hr|fieldset|legend|section' + '|article|aside|hgroup|header|footer|nav|figure|figcaption|details|menu|summary|asset|raw-code';

  if (_typeof(options) === 'object') {
    if (options.hasOwnProperty('preserve_linebreaks')) {
      preserve_linebreaks = options.preserve_linebreaks;
    }

    if (options.hasOwnProperty('preserve_br')) {
      preserve_br = options.preserve_br;
    }

    if (options.hasOwnProperty('blocks')) {
      blocklist = options.blocks;
    }

    if (options.hasOwnProperty('append_blocks')) {
      blocklist += '|' + options.append_blocks;
    }
  } // Normalize line breaks.


  text = text.replace(/\r\n|\r/g, '\n'); // Remove line breaks from <object>.

  if (text.indexOf('<object') !== -1) {
    text = text.replace(/<object[\s\S]+?<\/object>/g, function (a) {
      return a.replace(/\n+/g, '');
    });
  } // Remove line breaks from tags.


  text = text.replace(/<[^<>]+>/g, function (a) {
    return a.replace(/[\n\t ]+/g, ' ');
  }); // Preserve line breaks in <pre> and <script> tags.

  if (text.indexOf('<pre') !== -1 || text.indexOf('<script') !== -1) {
    preserve_linebreaks = true;
    text = text.replace(/<(pre|script|raw-code)[^>]*>[\s\S]*?<\/\1>/g, function (a) {
      return a.replace(/\n/g, '<wp-line-break>');
    });
  }

  if (text.indexOf('<figcaption') !== -1) {
    text = text.replace(/\s*(<figcaption[^>]*>)/g, '$1');
    text = text.replace(/<\/figcaption>\s*/g, '</figcaption>');
  } // Keep <br> tags inside captions.


  if (text.indexOf('[caption') !== -1) {
    preserve_br = true;
    text = text.replace(/\[caption[\s\S]+?\[\/caption\]/g, function (a) {
      a = a.replace(/<br([^>]*)>/g, '<wp-temp-br$1>');
      a = a.replace(/<[^<>]+>/g, function (b) {
        return b.replace(/[\n\t ]+/, ' ');
      });
      return a.replace(/\s*\n\s*/g, '<wp-temp-br />');
    });
  }

  text = text + '\n\n';
  text = text.replace(/<br \/>\s*<br \/>/gi, '\n\n'); // Pad block tags with two line breaks.

  text = text.replace(new RegExp('(<(?:' + blocklist + ')(?: [^>]*)?>)', 'gi'), '\n\n$1');
  text = text.replace(new RegExp('(</(?:' + blocklist + ')>)', 'gi'), '$1\n\n');
  text = text.replace(/<hr( [^>]*)?>/gi, '<hr$1>\n\n'); // Remove white space chars around <option>.

  text = text.replace(/\s*<option/gi, '<option');
  text = text.replace(/<\/option>\s*/gi, '</option>'); // Normalize multiple line breaks and white space chars.

  text = text.replace(/\n\s*\n+/g, '\n\n'); // Convert two line breaks to a paragraph.

  text = text.replace(/([\s\S]+?)\n\n/g, '<p>$1</p>\n'); // Remove empty paragraphs.

  text = text.replace(/<p>\s*?<\/p>/gi, ''); // Remove <p> tags that are around block tags.

  text = text.replace(new RegExp('<p>\\s*(</?(?:' + blocklist + ')(?: [^>]*)?>)\\s*</p>', 'gi'), '$1');
  text = text.replace(/<p>(<li.+?)<\/p>/gi, '$1'); // Fix <p> in blockquotes.

  text = text.replace(/<p>\s*<blockquote([^>]*)>/gi, '<blockquote$1><p>');
  text = text.replace(/<\/blockquote>\s*<\/p>/gi, '</p></blockquote>'); // Remove <p> tags that are wrapped around block tags.

  text = text.replace(new RegExp('<p>\\s*(</?(?:' + blocklist + ')(?: [^>]*)?>)', 'gi'), '$1');
  text = text.replace(new RegExp('(</?(?:' + blocklist + ')(?: [^>]*)?>)\\s*</p>', 'gi'), '$1');
  text = text.replace(/(<br[^>]*>)\s*\n/gi, '$1'); // Add <br> tags.

  text = text.replace(/\s*\n/g, '<br />\n'); // Remove <br> tags that are around block tags.

  text = text.replace(new RegExp('(</?(?:' + blocklist + ')[^>]*>)\\s*<br />', 'gi'), '$1');
  text = text.replace(/<br \/>(\s*<\/?(?:p|li|div|dl|dd|dt|th|pre|td|ul|ol)>)/gi, '$1'); // Remove <p> and <br> around captions.

  text = text.replace(/(?:<p>|<br ?\/?>)*\s*\[caption([^\[]+)\[\/caption\]\s*(?:<\/p>|<br ?\/?>)*/gi, '[caption$1[/caption]'); // Make sure there is <p> when there is </p> inside block tags that can contain other blocks.

  text = text.replace(/(<(?:div|th|td|form|fieldset|dd)[^>]*>)(.*?)<\/p>/g, function (a, b, c) {
    if (c.match(/<p( [^>]*)?>/)) {
      return a;
    }

    return b + '<p>' + c + '</p>';
  }); // Restore the line breaks in <pre> and <script> tags.

  if (preserve_linebreaks) {
    text = text.replace(/<wp-line-break>/g, '\n');
  } // Restore the <br> tags in captions.


  if (preserve_br) {
    text = text.replace(/<wp-temp-br([^>]*)>/g, '<br$1>');
  }

  return text;
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

var defineProperty = defineProperty$1;

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;

    defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var script = {
  name: 'LinkButton',
  components: {
    EditorIcon: EditorIcon,
    EditorDropButton: EditorDropButton
  },
  props: {
    codeMirror: {
      type: null,
      required: true
    },
    icon: String,
    label: String
  },
  data: function data() {
    return {
      link: ''
    };
  },
  methods: {
    setLink: function setLink() {
      this.link = '';
    },
    insert: function insert() {
      if (this.link) {
        CMWrapCommand(this.codeMirror, '<a href="' + this.link + '">', '</a>');
      }

      this.$refs.button.close();
    }
  }
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier
/* server only */
, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
  if (typeof shadowMode !== 'boolean') {
    createInjectorSSR = createInjector;
    createInjector = shadowMode;
    shadowMode = false;
  } // Vue.extend constructor export interop.


  var options = typeof script === 'function' ? script.options : script; // render functions

  if (template && template.render) {
    options.render = template.render;
    options.staticRenderFns = template.staticRenderFns;
    options._compiled = true; // functional template

    if (isFunctionalTemplate) {
      options.functional = true;
    }
  } // scopedId


  if (scopeId) {
    options._scopeId = scopeId;
  }

  var hook;

  if (moduleIdentifier) {
    // server build
    hook = function hook(context) {
      // 2.3 injection
      context = context || // cached call
      this.$vnode && this.$vnode.ssrContext || // stateful
      this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext; // functional
      // 2.2 with runInNewContext: true

      if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
        context = __VUE_SSR_CONTEXT__;
      } // inject component styles


      if (style) {
        style.call(this, createInjectorSSR(context));
      } // register component module identifier for async chunk inference


      if (context && context._registeredComponents) {
        context._registeredComponents.add(moduleIdentifier);
      }
    }; // used by ssr in case component is cached and beforeCreate
    // never gets called


    options._ssrRegister = hook;
  } else if (style) {
    hook = shadowMode ? function () {
      style.call(this, createInjectorShadow(this.$root.$options.shadowRoot));
    } : function (context) {
      style.call(this, createInjector(context));
    };
  }

  if (hook) {
    if (options.functional) {
      // register for functional component in vue file
      var originalRender = options.render;

      options.render = function renderWithStyleInjection(h, context) {
        hook.call(context);
        return originalRender(h, context);
      };
    } else {
      // inject component registration as beforeCreate hook
      var existing = options.beforeCreate;
      options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
    }
  }

  return script;
}

var normalizeComponent_1 = normalizeComponent;

/* script */
const __vue_script__ = script;
/* template */
var __vue_render__ = function () {var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('editor-drop-button',{ref:"button",staticClass:"editor-button-link",attrs:{"icon":_vm.icon,"label":_vm.label},on:{"open":_vm.setLink}},[_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.link),expression:"link"}],attrs:{"type":"url","placeholder":"link"},domProps:{"value":(_vm.link)},on:{"input":function($event){if($event.target.composing){ return; }_vm.link=$event.target.value;}}}),_vm._v(" "),_c('button',{on:{"click":_vm.insert}},[_c('editor-icon',{attrs:{"icon":"check"}})],1)])};
var __vue_staticRenderFns__ = [];

  /* style */
  const __vue_inject_styles__ = undefined;
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  

  
  var LinkButton = normalizeComponent_1(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    undefined,
    undefined
  );

function CMWrapCommand(codeMirror, open, close) {
  var text = codeMirror.getSelection();
  var replace = open + text + close;
  codeMirror.replaceSelection(replace);
  codeMirror.focus();

  if (!text.length) {
    setTimeout(function () {
      var cursorPos = codeMirror.getCursor();
      var search = codeMirror.getSearchCursor(close, cursorPos);

      if (search.findPrevious()) {
        codeMirror.setCursor(search.pos.from);
      }
    });
  }
}
var WrapCommand =
/*#__PURE__*/
function () {
  function WrapCommand(open, close) {
    _classCallCheck(this, WrapCommand);

    this._open = open;
    this._close = close;
  }

  _createClass(WrapCommand, [{
    key: "exec",
    value: function exec(codeMirror) {
      CMWrapCommand(codeMirror, this._open, this._close);
    }
  }]);

  return WrapCommand;
}();

var Commands =
/*#__PURE__*/
function () {
  function Commands() {
    _classCallCheck(this, Commands);

    this._commands = {};
  }

  _createClass(Commands, [{
    key: "exists",
    value: function exists(name) {
      return this._commands.hasOwnProperty(name);
    }
  }, {
    key: "add",
    value: function add(name, icon, label, command) {
      this._commands[name] = {
        icon: icon,
        label: label,
        command: command
      };
      return this;
    }
  }, {
    key: "addComponent",
    value: function addComponent(name, icon, label, component) {
      this._commands[name] = {
        icon: icon,
        label: label,
        component: component
      };
      return this;
    }
  }, {
    key: "get",
    value: function get(name) {
      return this._commands[name];
    }
  }]);

  return Commands;
}();

var commands = new Commands();
commands.add('bold', 'bold', 'Bold', new WrapCommand('<strong>', '</strong>'));
commands.add('italic', 'italic', 'Italic', new WrapCommand('<em>', '</em>'));
commands.add('underline', 'underline', 'Underline', new WrapCommand('<u>', '</u>'));
commands.add('strike', 'strike', 'Strike through', new WrapCommand('<del>', '</del>'));
commands.addComponent('link', 'link', 'Link', LinkButton);
commands.add('blockquote', 'blockquote', 'Blockquote', new WrapCommand('<blockquote>', '</blockquote>'));

var script$1 = {
  name: 'EditorToolbar',
  components: {
    EditorButton: EditorButton
  },
  props: {
    items: {
      type: String,
      required: true
    },
    codeMirror: {
      type: null,
      required: true
    }
  },
  computed: {
    buttons: function buttons() {
      var items = this.items.split(' ').map(function (item) {
        return item.trim();
      }).filter(function (item) {
        return item.length > 0;
      });
      var buttons = [];
      items.forEach(function (item) {
        if (item === '|') {
          buttons.push('|');
        } else {
          var command = commands.get(item);

          if (command) {
            buttons.push(command);
          }
        }
      });
      console.log(buttons);
      return buttons;
    }
  },
  methods: {
    doCommand: function doCommand(button) {
      button.command.exec(this.codeMirror);
    }
  }
};

/* script */
const __vue_script__$1 = script$1;
/* template */
var __vue_render__$1 = function () {var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return (_vm.codeMirror)?_c('div',{staticClass:"lede-code-toolbar"},[_vm._l((_vm.buttons),function(button,idx){return [(button === '|')?_c('span',{key:idx,staticClass:"toolbar-sep"}):(button.component)?_c(button.component,{key:idx,tag:"component",attrs:{"code-mirror":_vm.codeMirror,"label":button.label,"icon":button.icon}}):_c('editor-button',{key:idx,attrs:{"icon":button.icon},on:{"click":function($event){return _vm.doCommand(button)}}})]})],2):_vm._e()};
var __vue_staticRenderFns__$1 = [];

  /* style */
  const __vue_inject_styles__$1 = undefined;
  /* scoped */
  const __vue_scope_id__$1 = "data-v-e0ae4a38";
  /* module identifier */
  const __vue_module_identifier__$1 = undefined;
  /* functional template */
  const __vue_is_functional_template__$1 = false;
  /* style inject */
  
  /* style inject SSR */
  

  
  var EditorToolbar = normalizeComponent_1(
    { render: __vue_render__$1, staticRenderFns: __vue_staticRenderFns__$1 },
    __vue_inject_styles__$1,
    __vue_script__$1,
    __vue_scope_id__$1,
    __vue_is_functional_template__$1,
    __vue_module_identifier__$1,
    undefined,
    undefined
  );

require('codemirror/lib/codemirror.css');

require('codemirror/mode/htmlmixed/htmlmixed');
var defaultConfig = {
  lineNumbers: true,
  mode: 'htmlmixed',
  matchTags: {
    bothTags: true
  },
  autoCloseTags: true,
  lineWrapping: true,
  extraKeys: {
    'Ctrl-J': 'toMatchingTag'
  }
};
var script$2 = {
  name: 'LedeCodeEditor',
  components: {
    EditorToolbar: EditorToolbar
  },
  props: {
    config: {
      type: Object,
      default: function _default() {
        return {};
      }
    },
    value: {
      type: String
    },
    autoP: {
      type: [Boolean, Object]
    },
    toolbar: {
      type: String,
      default: 'bold  italic  underline  strike  |  link blockquote'
    }
  },
  computed: {
    code: function code() {
      return this.value;
    }
  },
  watch: {
    value: function value(val) {
      this.setValue(val);
    }
  },
  data: function data() {
    return {
      codeMirror: null
    };
  },
  mounted: function mounted() {
    this.init();
  },
  destroy: function destroy() {
    // garbage cleanup
    var element = this.codeMirror.doc.codeMirror.getWrapperElement();
    element && element.remove && element.remove();
  },
  methods: {
    init: function init() {
      var _this = this;

      var config = Object.assign(this.config, defaultConfig);
      this.codeMirror = CodeMirror.fromTextArea(this.$refs.textarea, config);
      this.codeMirror.on('change', function (codeMirror) {
        if (_this.$emit) {
          _this.$emit('update', codeMirror.getValue());
        }
      });
      this.codeMirror.on('focus', function () {
        _this.$emit('focus');
      });
      this.codeMirror.on('blur', function () {
        _this.$emit('blur');
      });
    },
    focus: function focus(atBeginning) {
      if (!this.codeMirror) {
        return;
      }

      this.codeMirror.focus();

      if (!atBeginning) {
        this.codeMirror.setCursor(this.codeMirror.lineCount(), 0);
      }
    },
    setValue: function setValue(value) {
      if (this.codeMirror) {
        this.codeMirror.setValue(value);
      }
    },
    getValue: function getValue() {
      var content = this.codeMirror.getValue();
      return this.autoP ? AutoP(content, this.autoP) : content;
    },
    startResize: function startResize(e) {
      function height_of($el) {
        return parseInt(window.getComputedStyle($el).height.replace(/px$/, ''));
      }

      var MIN_HEIGHT = 200;
      var start_y;
      var start_h;
      var cm = this.codeMirror;

      function on_drag(e) {
        cm.setSize(null, Math.max(MIN_HEIGHT, start_h + e.y - start_y) + 'px');
      }

      function on_release() {
        document.body.removeEventListener('mousemove', on_drag);
        window.removeEventListener('mouseup', on_release);
      }

      start_x = e.x;
      start_y = e.y;
      start_h = height_of(this.$refs.container);
      document.body.addEventListener('mousemove', on_drag);
      window.addEventListener('mouseup', on_release);
    }
  }
};

/* script */
const __vue_script__$2 = script$2;
/* template */
var __vue_render__$2 = function () {var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{staticClass:"lede-code-editor"},[(_vm.toolbar)?_c('editor-toolbar',{attrs:{"items":_vm.toolbar,"code-mirror":_vm.codeMirror}}):_vm._e(),_vm._v(" "),_c('div',{ref:"container",staticClass:"code-editor-container"},[_c('textarea',{ref:"textarea",staticClass:"lede-code-editor"})]),_vm._v(" "),_c('div',{ref:"resize",staticClass:"code-editor-resize",on:{"mousedown":_vm.startResize}})],1)};
var __vue_staticRenderFns__$2 = [];

  /* style */
  const __vue_inject_styles__$2 = undefined;
  /* scoped */
  const __vue_scope_id__$2 = undefined;
  /* module identifier */
  const __vue_module_identifier__$2 = undefined;
  /* functional template */
  const __vue_is_functional_template__$2 = false;
  /* style inject */
  
  /* style inject SSR */
  

  
  var component = normalizeComponent_1(
    { render: __vue_render__$2, staticRenderFns: __vue_staticRenderFns__$2 },
    __vue_inject_styles__$2,
    __vue_script__$2,
    __vue_scope_id__$2,
    __vue_is_functional_template__$2,
    __vue_module_identifier__$2,
    undefined,
    undefined
  );

export default component;
export { AutoP, CMWrapCommand, commands as Commands, WrapCommand };
